import 'dart:math';

import 'package:random/models/game/game_board.dart';
import 'package:random/models/game/game_cell.dart';
import 'package:random/models/game/game_settings.dart';

class Game {
  GameBoard board;
  GameSettings settings;
  bool isRunning = false;
  bool isFinished = false;
  int availableBlocksCount = 0;
  int movesCount = 0;
  int score = 0;

  Game({
    required this.board,
    required this.settings,
    this.isRunning = false,
  });

  factory Game.createNull() {
    return Game(
      board: GameBoard.createNull(),
      settings: GameSettings.createDefault(),
    );
  }

  factory Game.createNew({GameSettings? gameSettings}) {
    GameSettings settings = gameSettings ?? GameSettings.createDefault();

    return Game(
      board: GameBoard.createRandom(settings),
      settings: settings,
      isRunning: true,
    );
  }

  void stop() {
    isRunning = false;
    isFinished = true;
  }

  GameCell getCell(int x, int y) {
    return board.cells[y][x];
  }

  int? getCellValue(int x, int y) {
    return getCell(x, y).value;
  }

  void updateCellValue(int x, int y, int? value) {
    board.cells[y][x].value = value;
  }

  void setRandomCellValue(int x, int y, GameSettings settings) {
    final int maxValue = settings.colorsCount;
    final rand = Random();
    int value = 1 + rand.nextInt(maxValue);

    board.cells[y][x].value = value;
  }

  void increaseMovesCount() {
    movesCount += 1;
  }

  void increaseScore(int? count) {
    score += (count ?? 0);
  }

  @override
  String toString() {
    return 'Game(${toJson()})';
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      'board': board.toJson(),
      'settings': settings.toJson(),
      'isRunning': isRunning,
      'isFinished': isFinished,
      'availableBlocksCount': availableBlocksCount,
      'movesCount': movesCount,
      'score': score,
    };
  }

  void dump() {
    GameBoard.printGrid(board.cells);
    print(settings.toJson());
    print(toJson());
  }
}
